/* Generated by wayland-scanner 1.25.0 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

extern const struct wl_interface wl_output_interface;
extern const struct wl_interface wl_surface_interface;
extern const struct wl_interface zwp_fullscreen_shell_mode_feedback_v1_interface;

static const struct wl_interface *fullscreen_shell_unstable_v1_types[] = {
	NULL,
	&wl_surface_interface,
	NULL,
	&wl_output_interface,
	&wl_surface_interface,
	&wl_output_interface,
	NULL,
	&zwp_fullscreen_shell_mode_feedback_v1_interface,
};

static const struct wl_message zwp_fullscreen_shell_v1_requests[] = {
	{ "release", "", fullscreen_shell_unstable_v1_types + 0 },
	{ "present_surface", "?ou?o", fullscreen_shell_unstable_v1_types + 1 },
	{ "present_surface_for_mode", "ooin", fullscreen_shell_unstable_v1_types + 4 },
};

static const struct wl_message zwp_fullscreen_shell_v1_events[] = {
	{ "capability", "u", fullscreen_shell_unstable_v1_types + 0 },
};

WL_EXPORT const struct wl_interface zwp_fullscreen_shell_v1_interface = {
	"zwp_fullscreen_shell_v1", 1,
	3, zwp_fullscreen_shell_v1_requests,
	1, zwp_fullscreen_shell_v1_events,
};

static const struct wl_message zwp_fullscreen_shell_mode_feedback_v1_events[] = {
	{ "mode_successful", "", fullscreen_shell_unstable_v1_types + 0 },
	{ "mode_failed", "", fullscreen_shell_unstable_v1_types + 0 },
	{ "present_cancelled", "", fullscreen_shell_unstable_v1_types + 0 },
};

WL_EXPORT const struct wl_interface zwp_fullscreen_shell_mode_feedback_v1_interface = {
	"zwp_fullscreen_shell_mode_feedback_v1", 1,
	0, NULL,
	3, zwp_fullscreen_shell_mode_feedback_v1_events,
};

